@@ -28,9 +28,11 @@
 
 #include <iostream>
 
-#ifdef HAVE_LIBNGTCP2_CRYPTO_QUICTLS
+#if defined(HAVE_LIBNGTCP2_CRYPTO_QUICTLS) ||                                  \
+  defined(HAVE_LIBNGTCP2_CRYPTO_LIBRESSL)
 #  include <ngtcp2/ngtcp2_crypto_quictls.h>
-#endif // HAVE_LIBNGTCP2_CRYPTO_QUICTLS
+#endif // defined(HAVE_LIBNGTCP2_CRYPTO_QUICTLS) ||
+       // defined(HAVE_LIBNGTCP2_CRYPTO_LIBRESSL)
 #ifdef HAVE_LIBNGTCP2_CRYPTO_BORINGSSL
 #  include <ngtcp2/ngtcp2_crypto_boringssl.h>
 #endif // HAVE_LIBNGTCP2_CRYPTO_BORINGSSL
@@ -369,45 +371,27 @@ int Client::quic_init(const sockaddr *local_addr, socklen_t local_addrlen,
   }
 
   auto callbacks = ngtcp2_callbacks{
-    ngtcp2_crypto_client_initial_cb,
-    nullptr, // recv_client_initial
-    ngtcp2_crypto_recv_crypto_data_cb,
-    h2load::handshake_completed,
-    nullptr, // recv_version_negotiation
-    ngtcp2_crypto_encrypt_cb,
-    ngtcp2_crypto_decrypt_cb,
-    ngtcp2_crypto_hp_mask_cb,
-    h2load::recv_stream_data,
-    h2load::acked_stream_data_offset,
-    nullptr, // stream_open
-    h2load::stream_close,
-    nullptr, // recv_stateless_reset
-    ngtcp2_crypto_recv_retry_cb,
-    h2load::extend_max_local_streams_bidi,
-    nullptr, // extend_max_local_streams_uni
-    h2load::rand,
-    get_new_connection_id,
-    nullptr, // remove_connection_id
-    ngtcp2_crypto_update_key_cb,
-    nullptr, // path_validation
-    nullptr, // select_preferred_addr
-    h2load::stream_reset,
-    nullptr, // extend_max_remote_streams_bidi
-    nullptr, // extend_max_remote_streams_uni
-    h2load::extend_max_stream_data,
-    nullptr, // dcid_status
-    nullptr, // handshake_confirmed
-    nullptr, // recv_new_token
-    ngtcp2_crypto_delete_crypto_aead_ctx_cb,
-    ngtcp2_crypto_delete_crypto_cipher_ctx_cb,
-    nullptr, // recv_datagram
-    nullptr, // ack_datagram
-    nullptr, // lost_datagram
-    ngtcp2_crypto_get_path_challenge_data_cb,
-    h2load::stream_stop_sending,
-    nullptr, // version_negotiation
-    h2load::recv_rx_key,
-    nullptr, // recv_tx_key
+    .client_initial = ngtcp2_crypto_client_initial_cb,
+    .recv_crypto_data = ngtcp2_crypto_recv_crypto_data_cb,
+    .handshake_completed = h2load::handshake_completed,
+    .encrypt = ngtcp2_crypto_encrypt_cb,
+    .decrypt = ngtcp2_crypto_decrypt_cb,
+    .hp_mask = ngtcp2_crypto_hp_mask_cb,
+    .recv_stream_data = h2load::recv_stream_data,
+    .acked_stream_data_offset = h2load::acked_stream_data_offset,
+    .stream_close = h2load::stream_close,
+    .recv_retry = ngtcp2_crypto_recv_retry_cb,
+    .extend_max_local_streams_bidi = h2load::extend_max_local_streams_bidi,
+    .rand = h2load::rand,
+    .get_new_connection_id = get_new_connection_id,
+    .update_key = ngtcp2_crypto_update_key_cb,
+    .stream_reset = h2load::stream_reset,
+    .extend_max_stream_data = h2load::extend_max_stream_data,
+    .delete_crypto_aead_ctx = ngtcp2_crypto_delete_crypto_aead_ctx_cb,
+    .delete_crypto_cipher_ctx = ngtcp2_crypto_delete_crypto_cipher_ctx_cb,
+    .get_path_challenge_data = ngtcp2_crypto_get_path_challenge_data_cb,
+    .stream_stop_sending = h2load::stream_stop_sending,
+    .recv_rx_key = h2load::recv_rx_key,
   };
 
   ngtcp2_cid scid, dcid;
@@ -582,19 +566,21 @@ int Client::read_quic() {
   sockaddr_union su;
   int rv;
   size_t pktcnt = 0;
-  ngtcp2_pkt_info pi{};
+  ngtcp2_pkt_info pi;
 
-  iovec msg_iov;
-  msg_iov.iov_base = buf.data();
-  msg_iov.iov_len = buf.size();
-
-  msghdr msg{};
-  msg.msg_name = &su;
-  msg.msg_iov = &msg_iov;
-  msg.msg_iovlen = 1;
+  iovec msg_iov{
+    .iov_base = buf.data(),
+    .iov_len = buf.size(),
+  };
 
   uint8_t msg_ctrl[CMSG_SPACE(sizeof(int))];
-  msg.msg_control = msg_ctrl;
+
+  msghdr msg{
+    .msg_name = &su,
+    .msg_iov = &msg_iov,
+    .msg_iovlen = 1,
+    .msg_control = msg_ctrl,
+  };
 
   auto ts = quic_timestamp();
 
@@ -670,6 +656,80 @@ int Client::read_quic() {
   return 0;
 }
 
+namespace {
+ngtcp2_ssize write_pkt(ngtcp2_conn *conn, ngtcp2_path *path,
+                       ngtcp2_pkt_info *pi, uint8_t *dest, size_t destlen,
+                       ngtcp2_tstamp ts, void *user_data) {
+  auto c = static_cast<Client *>(user_data);
+
+  return c->write_quic_pkt(path, pi, dest, destlen, ts);
+}
+} // namespace
+
+ngtcp2_ssize Client::write_quic_pkt(ngtcp2_path *path, ngtcp2_pkt_info *pi,
+                                    uint8_t *dest, size_t destlen,
+                                    ngtcp2_tstamp ts) {
+  std::array<nghttp3_vec, 16> vec;
+  auto s = static_cast<Http3Session *>(session.get());
+
+  for (;;) {
+    int64_t stream_id = -1;
+    int fin = 0;
+    ssize_t sveccnt = 0;
+
+    if (session && ngtcp2_conn_get_max_data_left(quic.conn)) {
+      sveccnt = s->write_stream(stream_id, fin, vec.data(), vec.size());
+      if (sveccnt == -1) {
+        return NGTCP2_ERR_CALLBACK_FAILURE;
+      }
+    }
+
+    ngtcp2_ssize ndatalen;
+    auto v = vec.data();
+    auto vcnt = static_cast<size_t>(sveccnt);
+
+    uint32_t flags =
+      NGTCP2_WRITE_STREAM_FLAG_MORE | NGTCP2_WRITE_STREAM_FLAG_PADDING;
+    if (fin) {
+      flags |= NGTCP2_WRITE_STREAM_FLAG_FIN;
+    }
+
+    auto nwrite = ngtcp2_conn_writev_stream(
+      quic.conn, path, nullptr, dest, destlen, &ndatalen, flags, stream_id,
+      reinterpret_cast<const ngtcp2_vec *>(v), vcnt, ts);
+    if (nwrite < 0) {
+      switch (nwrite) {
+      case NGTCP2_ERR_STREAM_DATA_BLOCKED:
+        assert(ndatalen == -1);
+        s->block_stream(stream_id);
+        continue;
+      case NGTCP2_ERR_STREAM_SHUT_WR:
+        assert(ndatalen == -1);
+        s->shutdown_stream_write(stream_id);
+        continue;
+      case NGTCP2_ERR_WRITE_MORE:
+        assert(ndatalen >= 0);
+        if (s->add_write_offset(stream_id, as_unsigned(ndatalen)) != 0) {
+          return NGTCP2_ERR_CALLBACK_FAILURE;
+        }
+        continue;
+      }
+
+      ngtcp2_ccerr_set_liberr(&quic.last_error, static_cast<int>(nwrite),
+                              nullptr, 0);
+
+      return NGTCP2_ERR_CALLBACK_FAILURE;
+    }
+
+    if (ndatalen >= 0 &&
+        s->add_write_offset(stream_id, as_unsigned(ndatalen)) != 0) {
+      return NGTCP2_ERR_CALLBACK_FAILURE;
+    }
+
+    return nwrite;
+  }
+}
+
 int Client::write_quic() {
   int rv;
 
@@ -690,145 +750,47 @@ int Client::write_quic() {
     }
   }
 
-  std::array<nghttp3_vec, 16> vec;
-  auto max_udp_payload_size =
-    ngtcp2_conn_get_max_tx_udp_payload_size(quic.conn);
-  auto path_max_udp_payload_size =
-    ngtcp2_conn_get_path_max_tx_udp_payload_size(quic.conn);
-  auto txbuf = std::span{quic.tx.data.get(),
-                         std::max(ngtcp2_conn_get_send_quantum(quic.conn),
-                                  path_max_udp_payload_size)};
-  auto buf = txbuf;
+  auto txbuf = std::span{quic.tx.data.get(), QUIC_TX_DATALEN};
   ngtcp2_path_storage ps;
   size_t gso_size = 0;
-  auto pkt = std::span<const uint8_t>{};
-  auto extra_pkt = std::span<const uint8_t>{};
 
   ngtcp2_path_storage_zero(&ps);
 
-  auto s = static_cast<Http3Session *>(session.get());
-  auto ts = quic_timestamp();
-
-  for (;;) {
-    int64_t stream_id = -1;
-    int fin = 0;
-    ssize_t sveccnt = 0;
-
-    if (session && ngtcp2_conn_get_max_data_left(quic.conn)) {
-      sveccnt = s->write_stream(stream_id, fin, vec.data(), vec.size());
-      if (sveccnt == -1) {
-        return -1;
-      }
-    }
-
-    ngtcp2_ssize ndatalen;
-    auto v = vec.data();
-    auto vcnt = static_cast<size_t>(sveccnt);
-
-    uint32_t flags = NGTCP2_WRITE_STREAM_FLAG_MORE;
-    if (fin) {
-      flags |= NGTCP2_WRITE_STREAM_FLAG_FIN;
-    }
-
-    auto buflen = buf.size() >= max_udp_payload_size
-                    ? max_udp_payload_size
-                    : path_max_udp_payload_size;
-    auto nwrite = ngtcp2_conn_writev_stream(
-      quic.conn, &ps.path, nullptr, buf.data(), buflen, &ndatalen, flags,
-      stream_id, reinterpret_cast<const ngtcp2_vec *>(v), vcnt, ts);
-    if (nwrite < 0) {
-      switch (nwrite) {
-      case NGTCP2_ERR_STREAM_DATA_BLOCKED:
-        assert(ndatalen == -1);
-        s->block_stream(stream_id);
-        continue;
-      case NGTCP2_ERR_STREAM_SHUT_WR:
-        assert(ndatalen == -1);
-        s->shutdown_stream_write(stream_id);
-        continue;
-      case NGTCP2_ERR_WRITE_MORE:
-        assert(ndatalen >= 0);
-        if (s->add_write_offset(stream_id, as_unsigned(ndatalen)) != 0) {
-          return -1;
-        }
-        continue;
-      }
-
-      ngtcp2_ccerr_set_liberr(&quic.last_error, static_cast<int>(nwrite),
-                              nullptr, 0);
-      return -1;
-    } else if (ndatalen >= 0 &&
-               s->add_write_offset(stream_id, as_unsigned(ndatalen)) != 0) {
-      return -1;
-    }
-
-    quic_restart_pkt_timer();
-
-    if (nwrite == 0) {
-      pkt = std::span{std::ranges::begin(txbuf), std::ranges::begin(buf)};
-      if (pkt.empty()) {
-        return 0;
-      }
-
-      break;
-    }
-
-    auto last_pkt_pos = std::ranges::begin(buf);
-
-    buf = buf.subspan(as_unsigned(nwrite));
-
-    if (last_pkt_pos == std::ranges::begin(txbuf)) {
-      gso_size = as_unsigned(nwrite);
-    } else if (static_cast<size_t>(nwrite) > gso_size ||
-               (gso_size > path_max_udp_payload_size &&
-                static_cast<size_t>(nwrite) != gso_size)) {
-      pkt = std::span{std::ranges::begin(txbuf), last_pkt_pos};
-      extra_pkt = std::span{last_pkt_pos, std::ranges::begin(buf)};
-      break;
-    }
-
-    // Assume that the path does not change.
-    if (buf.size() < path_max_udp_payload_size ||
-        static_cast<size_t>(nwrite) < gso_size) {
-      pkt = std::span{std::ranges::begin(txbuf), std::ranges::begin(buf)};
-      break;
-    }
+  auto nwrite = ngtcp2_conn_write_aggregate_pkt(
+    quic.conn, &ps.path, nullptr, txbuf.data(), txbuf.size(), &gso_size,
+    h2load::write_pkt, quic_timestamp());
+  if (nwrite < 0) {
+    return -1;
   }
 
-  if (write_udp_or_blocked(ps.path, pkt, gso_size) != 0) {
-    if (!extra_pkt.empty()) {
-      on_send_blocked(ps.path.remote, extra_pkt, extra_pkt.size());
-    }
-  } else if (!extra_pkt.empty()) {
-    write_udp_or_blocked(ps.path, extra_pkt, extra_pkt.size());
+  quic_restart_pkt_timer();
+
+  if (nwrite == 0) {
+    return 0;
   }
 
-  ngtcp2_conn_update_pkt_tx_time(quic.conn, ts);
+  write_udp_or_blocked(ps.path, txbuf.first(static_cast<size_t>(nwrite)),
+                       gso_size);
 
   return 0;
 }
 
-int Client::write_udp_or_blocked(const ngtcp2_path &path,
-                                 std::span<const uint8_t> data,
-                                 size_t gso_size) {
+void Client::write_udp_or_blocked(const ngtcp2_path &path,
+                                  std::span<const uint8_t> data,
+                                  size_t gso_size) {
   auto rest = write_udp(path.remote.addr, path.remote.addrlen, data, gso_size);
   if (!rest.empty()) {
     on_send_blocked(path.remote, data, gso_size);
-
-    return -1;
   }
-
-  return 0;
 }
 
 void Client::on_send_blocked(const ngtcp2_addr &remote_addr,
                              std::span<const uint8_t> data, size_t gso_size) {
-  assert(quic.tx.num_blocked || !quic.tx.send_blocked);
-  assert(quic.tx.num_blocked < 2);
+  assert(!quic.tx.send_blocked);
 
   quic.tx.send_blocked = true;
 
-  auto &p = quic.tx.blocked[quic.tx.num_blocked++];
+  auto &p = quic.tx.blocked;
 
   memcpy(&p.remote_addr.su, remote_addr.addr, remote_addr.addrlen);
 
@@ -842,24 +804,19 @@ void Client::on_send_blocked(const ngtcp2_addr &remote_addr,
 int Client::send_blocked_packet() {
   assert(quic.tx.send_blocked);
 
-  for (; quic.tx.num_blocked_sent < quic.tx.num_blocked;
-       ++quic.tx.num_blocked_sent) {
-    auto &p = quic.tx.blocked[quic.tx.num_blocked_sent];
+  auto &p = quic.tx.blocked;
 
-    auto rest =
-      write_udp(&p.remote_addr.su.sa, p.remote_addr.len, p.data, p.gso_size);
-    if (!rest.empty()) {
-      p.data = rest;
+  auto rest =
+    write_udp(&p.remote_addr.su.sa, p.remote_addr.len, p.data, p.gso_size);
+  if (!rest.empty()) {
+    p.data = rest;
 
-      signal_write();
+    signal_write();
 
-      return 0;
-    }
+    return 0;
   }
 
   quic.tx.send_blocked = false;
-  quic.tx.num_blocked = 0;
-  quic.tx.num_blocked_sent = 0;
 
   return 0;
 }
