@@ -49,24 +49,28 @@ QUICListener::~QUICListener() {
 }
 
 void QUICListener::on_read() {
-  sockaddr_union su;
+  Address remote_addr;
   std::array<uint8_t, 64_k> buf;
   size_t pktcnt = 0;
-  iovec msg_iov{buf.data(), buf.size()};
-
-  msghdr msg{};
-  msg.msg_name = &su;
-  msg.msg_iov = &msg_iov;
-  msg.msg_iovlen = 1;
+  iovec msg_iov{
+    .iov_base = buf.data(),
+    .iov_len = buf.size(),
+  };
 
   uint8_t msg_ctrl[CMSG_SPACE(sizeof(int)) + CMSG_SPACE(sizeof(in6_pktinfo)) +
                    CMSG_SPACE(sizeof(int))];
-  msg.msg_control = msg_ctrl;
+
+  msghdr msg{
+    .msg_name = &remote_addr.su,
+    .msg_iov = &msg_iov,
+    .msg_iovlen = 1,
+    .msg_control = msg_ctrl,
+  };
 
   auto quic_conn_handler = worker_->get_quic_connection_handler();
 
   for (; pktcnt < 10;) {
-    msg.msg_namelen = sizeof(su);
+    msg.msg_namelen = sizeof(remote_addr.su);
     msg.msg_controllen = sizeof(msg_ctrl);
 
     auto nread = recvmsg(faddr_->fd, &msg, 0);
@@ -81,15 +85,15 @@ void QUICListener::on_read() {
       continue;
     }
 
-    if (util::quic_prohibited_port(util::get_port(&su))) {
+    if (util::quic_prohibited_port(util::get_port(&remote_addr.su))) {
       ++pktcnt;
 
       continue;
     }
 
     Address local_addr{};
-    if (util::msghdr_get_local_addr(local_addr, &msg, su.storage.ss_family) !=
-        0) {
+    if (util::msghdr_get_local_addr(local_addr, &msg,
+                                    remote_addr.su.storage.ss_family) != 0) {
       ++pktcnt;
 
       continue;
@@ -98,7 +102,7 @@ void QUICListener::on_read() {
     util::set_port(local_addr, faddr_->port);
 
     ngtcp2_pkt_info pi{
-      .ecn = util::msghdr_get_ecn(&msg, su.storage.ss_family),
+      .ecn = util::msghdr_get_ecn(&msg, remote_addr.su.storage.ss_family),
     };
 
     auto gso_size = util::msghdr_get_udp_gro(&msg);
@@ -113,10 +117,12 @@ void QUICListener::on_read() {
 
       ++pktcnt;
 
+      remote_addr.len = msg.msg_namelen;
+
       if (LOG_ENABLED(INFO)) {
         LOG(INFO) << "QUIC received packet: local="
-                  << util::to_numeric_addr(&local_addr) << " remote="
-                  << util::to_numeric_addr(&su.sa, msg.msg_namelen)
+                  << util::to_numeric_addr(&local_addr)
+                  << " remote=" << util::to_numeric_addr(&remote_addr)
                   << " ecn=" << log::hex << pi.ecn << log::dec << " " << datalen
                   << " bytes";
       }
@@ -126,10 +132,6 @@ void QUICListener::on_read() {
         break;
       }
 
-      Address remote_addr;
-      remote_addr.su = su;
-      remote_addr.len = msg.msg_namelen;
-
       quic_conn_handler->handle_packet(faddr_, remote_addr, local_addr, pi,
                                        data.first(datalen));
 
