@@ -35,7 +35,14 @@
 #include <cinttypes>
 #include <cstdlib>
 
-#include <openssl/rand.h>
+#include "ssl_compat.h"
+
+#ifdef NGHTTP2_OPENSSL_IS_WOLFSSL
+#  include <wolfssl/options.h>
+#  include <wolfssl/openssl/rand.h>
+#else // !NGHTTP2_OPENSSL_IS_WOLFSSL
+#  include <openssl/rand.h>
+#endif // !NGHTTP2_OPENSSL_IS_WOLFSSL
 
 #include <ev.h>
 
@@ -65,7 +72,7 @@ namespace shrpx {
 namespace {
 void drop_privileges(
 #ifdef HAVE_NEVERBLEED
-    neverbleed_t *nb
+  neverbleed_t *nb
 #endif // HAVE_NEVERBLEED
 ) {
   std::array<char, STRERROR_BUFSIZE> errbuf;
@@ -74,11 +81,17 @@ void drop_privileges(
   if (getuid() == 0 && config->uid != 0) {
 #ifdef HAVE_NEVERBLEED
     if (nb) {
-      neverbleed_setuidgid(nb, config->user.c_str(), 1);
+      neverbleed_setuidgid(nb, config->user.data(), 1);
     }
 #endif // HAVE_NEVERBLEED
 
-    if (initgroups(config->user.c_str(), config->gid) != 0) {
+    if (initgroups(config->user.data(),
+#ifndef __APPLE__
+                   config->gid
+#else  // defined(__APPLE__)
+                   static_cast<int>(config->gid)
+#endif // defined(__APPLE__)
+                   ) != 0) {
       auto error = errno;
       LOG(FATAL) << "Could not change supplementary groups: "
                  << xsi_strerror(error, errbuf.data(), errbuf.size());
@@ -113,12 +126,6 @@ void graceful_shutdown(ConnectionHandler *conn_handler) {
   LOG(NOTICE) << "Graceful shutdown signal received";
 
   conn_handler->set_graceful_shutdown(true);
-
-  // TODO What happens for the connections not established in the
-  // kernel?
-  conn_handler->accept_pending_connection();
-  conn_handler->delete_acceptor();
-
   conn_handler->graceful_shutdown_worker();
 
   auto single_worker = conn_handler->get_single_worker();
@@ -167,7 +174,7 @@ void ipc_readcb(struct ev_loop *loop, ev_io *w, int revents) {
     nghttp2_Exit(EXIT_FAILURE);
   }
 
-  for (ssize_t i = 0; i < nread; ++i) {
+  for (size_t i = 0; i < as_unsigned(nread); ++i) {
     switch (buf[i]) {
     case SHRPX_IPC_GRACEFUL_SHUTDOWN:
       graceful_shutdown(conn_handler);
@@ -198,7 +205,7 @@ namespace {
 int generate_ticket_key(TicketKey &ticket_key) {
   ticket_key.cipher = get_config()->tls.ticket.cipher;
   ticket_key.hmac = EVP_sha256();
-  ticket_key.hmac_keylen = EVP_MD_size(ticket_key.hmac);
+  ticket_key.hmac_keylen = static_cast<size_t>(EVP_MD_size(ticket_key.hmac));
 
   assert(static_cast<size_t>(EVP_CIPHER_key_length(ticket_key.cipher)) <=
          ticket_key.data.enc_key.size());
@@ -242,13 +249,14 @@ void renew_ticket_key_cb(struct ev_loop *loop, ev_timer *w, int revents) {
     assert(!old_keys.empty());
 
     auto max_tickets =
-        static_cast<size_t>(std::chrono::duration_cast<std::chrono::hours>(
-                                get_config()->tls.session_timeout)
-                                .count());
+      static_cast<size_t>(std::chrono::duration_cast<std::chrono::hours>(
+                            get_config()->tls.session_timeout)
+                            .count());
 
     new_keys.resize(std::min(max_tickets, old_keys.size() + 1));
-    std::copy_n(std::begin(old_keys), new_keys.size() - 1,
-                std::begin(new_keys) + 1);
+    std::ranges::copy_n(std::ranges::begin(old_keys),
+                        as_signed(new_keys.size() - 1),
+                        std::ranges::begin(new_keys) + 1);
   } else {
     ticket_keys->keys.resize(1);
   }
@@ -367,14 +375,17 @@ void memcached_get_ticket_key_cb(struct ev_loop *loop, ev_timer *w,
       key.hmac = EVP_sha256();
       key.hmac_keylen = hmac_keylen;
 
-      std::copy_n(p, key.data.name.size(), std::begin(key.data.name));
-      p += key.data.name.size();
+      p = std::ranges::copy_n(p, key.data.name.size(),
+                              std::ranges::begin(key.data.name))
+            .in;
 
-      std::copy_n(p, enc_keylen, std::begin(key.data.enc_key));
-      p += enc_keylen;
+      p = std::ranges::copy_n(p, as_signed(enc_keylen),
+                              std::ranges::begin(key.data.enc_key))
+            .in;
 
-      std::copy_n(p, hmac_keylen, std::begin(key.data.hmac_key));
-      p += hmac_keylen;
+      p = std::ranges::copy_n(p, as_signed(hmac_keylen),
+                              std::ranges::begin(key.data.hmac_key))
+            .in;
 
       ticket_keys->keys.push_back(std::move(key));
     }
@@ -476,14 +487,9 @@ int worker_process_event_loop(WorkerProcessConfig *wpconf) {
 #ifdef ENABLE_HTTP3
   conn_handler->set_quic_ipc_fd(wpconf->quic_ipc_fd);
   conn_handler->set_quic_lingering_worker_processes(
-      wpconf->quic_lingering_worker_processes);
+    wpconf->quic_lingering_worker_processes);
 #endif // ENABLE_HTTP3
 
-  for (auto &addr : config->conn.listener.addrs) {
-    conn_handler->add_acceptor(
-        std::make_unique<AcceptHandler>(&addr, conn_handler.get()));
-  }
-
   MemchunkPool mcpool;
 
   ev_timer renew_ticket_key_timer;
@@ -499,9 +505,9 @@ int worker_process_event_loop(WorkerProcessConfig *wpconf) {
       }
 
       conn_handler->set_tls_ticket_key_memcached_dispatcher(
-          std::make_unique<MemcachedDispatcher>(
-              &ticketconf.memcached.addr, loop, ssl_ctx,
-              StringRef{memcachedconf.host}, &mcpool, gen));
+        std::make_unique<MemcachedDispatcher>(&ticketconf.memcached.addr, loop,
+                                              ssl_ctx, memcachedconf.host,
+                                              &mcpool, gen));
 
       ev_timer_init(&renew_ticket_key_timer, memcached_get_ticket_key_cb, 0.,
                     0.);
@@ -513,15 +519,15 @@ int worker_process_event_loop(WorkerProcessConfig *wpconf) {
       if (!ticketconf.files.empty()) {
         if (!ticketconf.cipher_given) {
           LOG(WARN)
-              << "It is strongly recommended to specify "
-                 "--tls-ticket-key-cipher=aes-128-cbc (or "
-                 "tls-ticket-key-cipher=aes-128-cbc in configuration file) "
-                 "when --tls-ticket-key-file is used for the smooth "
-                 "transition when the default value of --tls-ticket-key-cipher "
-                 "becomes aes-256-cbc";
+            << "It is strongly recommended to specify "
+               "--tls-ticket-key-cipher=aes-128-cbc (or "
+               "tls-ticket-key-cipher=aes-128-cbc in configuration file) "
+               "when --tls-ticket-key-file is used for the smooth "
+               "transition when the default value of --tls-ticket-key-cipher "
+               "becomes aes-256-cbc";
         }
         auto ticket_keys = read_tls_ticket_key_file(
-            ticketconf.files, ticketconf.cipher, EVP_sha256());
+          ticketconf.files, ticketconf.cipher, EVP_sha256());
         if (!ticket_keys) {
           LOG(WARN) << "Use internal session ticket key generator";
         } else {
@@ -559,27 +565,29 @@ int worker_process_event_loop(WorkerProcessConfig *wpconf) {
 
     auto &qkm = qkms->keying_materials.front();
 
-    if (RAND_bytes(qkm.reserved.data(), qkm.reserved.size()) != 1) {
+    if (RAND_bytes(qkm.reserved.data(),
+                   static_cast<nghttp2_ssl_rand_length_type>(
+                     qkm.reserved.size())) != 1) {
       LOG(ERROR) << "Failed to generate QUIC secret reserved data";
       return -1;
     }
 
-    if (RAND_bytes(qkm.secret.data(), qkm.secret.size()) != 1) {
+    if (RAND_bytes(qkm.secret.data(), static_cast<nghttp2_ssl_rand_length_type>(
+                                        qkm.secret.size())) != 1) {
       LOG(ERROR) << "Failed to generate QUIC secret";
       return -1;
     }
 
-    if (RAND_bytes(qkm.salt.data(), qkm.salt.size()) != 1) {
+    if (RAND_bytes(qkm.salt.data(), static_cast<nghttp2_ssl_rand_length_type>(
+                                      qkm.salt.size())) != 1) {
       LOG(ERROR) << "Failed to generate QUIC salt";
       return -1;
     }
   }
 
   for (auto &qkm : qkms->keying_materials) {
-    if (generate_quic_connection_id_encryption_key(
-            qkm.cid_encryption_key.data(), qkm.cid_encryption_key.size(),
-            qkm.secret.data(), qkm.secret.size(), qkm.salt.data(),
-            qkm.salt.size()) != 0) {
+    if (generate_quic_connection_id_encryption_key(qkm.cid_encryption_key,
+                                                   qkm.secret, qkm.salt) != 0) {
       LOG(ERROR) << "Failed to generate QUIC Connection ID encryption key";
       return -1;
     }
@@ -588,7 +596,7 @@ int worker_process_event_loop(WorkerProcessConfig *wpconf) {
     if (!EVP_EncryptInit_ex(qkm.cid_encryption_ctx, EVP_aes_128_ecb(), nullptr,
                             qkm.cid_encryption_key.data(), nullptr)) {
       LOG(ERROR)
-          << "Failed to initialize QUIC Connection ID encryption context";
+        << "Failed to initialize QUIC Connection ID encryption context";
       return -1;
     }
 
@@ -598,7 +606,7 @@ int worker_process_event_loop(WorkerProcessConfig *wpconf) {
     if (!EVP_DecryptInit_ex(qkm.cid_decryption_ctx, EVP_aes_128_ecb(), nullptr,
                             qkm.cid_encryption_key.data(), nullptr)) {
       LOG(ERROR)
-          << "Failed to initialize QUIC Connection ID decryption context";
+        << "Failed to initialize QUIC Connection ID decryption context";
       return -1;
     }
 
@@ -609,7 +617,7 @@ int worker_process_event_loop(WorkerProcessConfig *wpconf) {
 
   conn_handler->set_worker_ids(wpconf->worker_ids);
   conn_handler->set_quic_lingering_worker_processes(
-      wpconf->quic_lingering_worker_processes);
+    wpconf->quic_lingering_worker_processes);
 #endif // ENABLE_HTTP3
 
   if (config->single_thread) {
@@ -646,13 +654,21 @@ int worker_process_event_loop(WorkerProcessConfig *wpconf) {
 #endif // !NOTHREADS
   }
 
+  // UNIX domain sockets are copied in AcceptHandler.  No need to keep
+  // the original.
+  for (auto &addr : config->conn.listener.addrs) {
+    if (addr.host_unix) {
+      close(addr.fd);
+    }
+  }
+
 #if defined(ENABLE_HTTP3) && defined(HAVE_LIBBPF)
   conn_handler->unload_bpf_objects();
 #endif // defined(ENABLE_HTTP3) && defined(HAVE_LIBBPF)
 
   drop_privileges(
 #ifdef HAVE_NEVERBLEED
-      nb.get()
+    nb.get()
 #endif // HAVE_NEVERBLEED
   );
 
@@ -668,15 +684,6 @@ int worker_process_event_loop(WorkerProcessConfig *wpconf) {
   ev_io_start(loop, &quic_ipcev);
 #endif // ENABLE_HTTP3
 
-  if (tls::upstream_tls_enabled(config->conn) && !config->tls.ocsp.disabled) {
-    if (config->tls.ocsp.startup) {
-      conn_handler->set_enable_acceptor_on_ocsp_completion(true);
-      conn_handler->disable_acceptor();
-    }
-
-    conn_handler->proceed_next_cert_ocsp();
-  }
-
   if (LOG_ENABLED(INFO)) {
     LOG(INFO) << "Entering event loop";
   }
@@ -687,8 +694,6 @@ int worker_process_event_loop(WorkerProcessConfig *wpconf) {
 
   ev_run(loop, 0);
 
-  conn_handler->cancel_ocsp_update();
-
   // Destroy SSL_CTX held in conn_handler before killing neverbleed
   // daemon.  Otherwise priv_rsa_finish yields "write error" and
   // worker process aborts.
